# frozen_string_literal: true

module Security
  module Orchestration
    class AssignService < ::BaseContainerService
      def execute
        validate_access!

        return error(_("Security policy project is already assigned.")) if policy_project_assigned?

        if policy_project_inherited?
          return error(_("You don't need to link the security policy projects from the group. " \
                         "All policies in the security policy projects are inherited already."))
        end

        if create_or_update_security_policy_configuration
          unassign_redundant_configurations if group_container?

          success
        end
      rescue ActiveRecord::RecordNotFound => _
        error(_('Policy project doesn\'t exist'))
      rescue ActiveRecord::RecordInvalid => _
        error(_('Couldn\'t assign policy to project or group'))
      end

      private

      def validate_access!
        return if current_user.can?(:update_security_orchestration_policy_project, container)

        raise Gitlab::Access::AccessDeniedError
      end

      def create_or_update_security_policy_configuration
        return unassign_policy_project if unassigning? && has_existing_policy?

        policy_project = Project.find(policy_project_id)

        return reassign_policy_project(policy_project) if has_existing_policy?

        assign_policy_project(policy_project)
      end

      def assign_policy_project(policy_project)
        create_result = container.create_security_orchestration_policy_configuration! do |p|
          p.security_policy_management_project_id = policy_project.id
        end

        if create_result
          create_security_policy_project_bot

          audit(policy_project, "Linked #{policy_project.name} as the security policy project")
          Security::SyncScanPoliciesWorker.perform_async(container.security_orchestration_policy_configuration.id)
        end

        create_result
      end

      def unassign_policy_project
        result = UnassignService.new(container: container, current_user: current_user).execute

        result.success?
      end

      def reassign_policy_project(policy_project)
        configuration = container.security_orchestration_policy_configuration
        old_policy_project = configuration.security_policy_management_project

        update_result = nil
        configuration.transaction do
          configuration.delete_scan_finding_rules
          update_result = configuration.update!(security_policy_management_project_id: policy_project.id)
        end

        if update_result
          audit(
            policy_project,
            "Changed the linked security policy project from #{old_policy_project.name} to #{policy_project.name}"
          )

          Security::SyncScanPoliciesWorker.perform_async(configuration.id)
        end

        update_result
      end

      def audit(policy_project, message)
        ::Gitlab::Audit::Auditor.audit(
          name: 'policy_project_updated',
          author: current_user,
          scope: container,
          target: policy_project,
          message: message
        )
      end

      def success
        ServiceResponse.success(payload: { policy_project: policy_project_id })
      end

      def error(message)
        ServiceResponse.error(payload: { policy_project: policy_project_id }, message: message)
      end

      def has_existing_policy?
        container.security_orchestration_policy_configuration.present?
      end

      def policy_project_id
        params[:policy_project_id]
      end

      def create_security_policy_project_bot
        if container.is_a?(Project)
          Security::OrchestrationConfigurationCreateBotWorker.perform_async(container.id, current_user.id)
        else
          create_bot_for_namespace
        end
      end

      def create_bot_for_namespace
        if Feature.enabled?(:security_policy_bot_worker, container)
          Security::OrchestrationConfigurationCreateBotForNamespaceWorker.perform_async(container.id, current_user.id)
        else
          container.security_orchestration_policy_configuration.all_project_ids.each do |project_id|
            Security::OrchestrationConfigurationCreateBotWorker.perform_async(project_id, current_user.id)
          end
        end
      end

      def unassign_redundant_configurations
        ::Security::UnassignRedundantPolicyConfigurationsWorker
          .perform_async(container.id, policy_project_id, current_user.id)
      end

      def policy_project_assigned?
        attrs = if project_container?
                  { project_id: container.id, security_policy_management_project_id: policy_project_id }
                else
                  { namespace_id: container.id, security_policy_management_project_id: policy_project_id }
                end

        # rubocop:disable CodeReuse/ActiveRecord -- Not suitable for a scope
        Security::OrchestrationPolicyConfiguration.exists?(attrs)
        # rubocop:enable CodeReuse/ActiveRecord
      end

      def policy_project_inherited?
        all_effective_policy_project_ids.include?(policy_project_id)
      end

      def all_effective_policy_project_ids
        # rubocop:disable Database/AvoidUsingPluckWithoutLimit -- plucking an Array
        # rubocop:disable CodeReuse/ActiveRecord -- plucking an Array
        container
          .all_security_orchestration_policy_configurations(include_invalid: true)
          .pluck(:security_policy_management_project_id)
        # rubocop:enable Database/AvoidUsingPluckWithoutLimit
        # rubocop:enable CodeReuse/ActiveRecord
      end

      def unassigning?
        policy_project_id.blank?
      end
    end
  end
end
