# frozen_string_literal: true

module Gitlab
  module Llm
    module Utils
      class CodeSuggestionFormatter
        PROMPT =
          <<~MESSAGE.chomp
            When you are responding with a code suggestion, format your code suggestion as follows:
              - Use the following format:
                <from>
                  [existing lines that you are suggesting to change]
                </from>
                <to>
                  [your suggestion]
                </to>
                - <from> tag must be identical to the lines as they appear in the diff, including any leading spaces or tabs
                - <to> tag must contain your suggestion
                - Opening and closing `<from>` and `<to>` tags should not be on the same line as the content
                - When making suggestions, always maintain the exact indentation as shown in the original diff. The suggestion should match the indentation of the line you are commenting on precisely, as it will be applied directly in place of the existing line.
                - Do note that you are given a raw diff so you must parse the raw diff before using them in your suggestion:
                  - Remove the first character of each line ('+', '-', or ' ') as it's part of the diff syntax, not the actual code.
                  - Use the remaining content for your suggestion, maintaining the original indentation.
          MESSAGE

        # NOTE: We might get multiple code suggestions on the same line as that's still valid so we should take
        #   that possibility of extra `<from>` into account here.
        #   Also, sometimes LLM returns tags inline like `<to>  some text</to>` for single line suggestions which
        #   we need to handle as well just in case.
        CODE_SUGGESTION_REGEX =
          %r{(.*?)^(?:<from>\n(.*?)^</from>\n<to>(.*?)^</to>|<from>(.+?)</from>\n<to>(.+?)</to>)(.*?)(?=<from>|\z)}m

        def self.append_prompt(body)
          return '' if body.blank?

          [body, PROMPT].join("\n\n") # Leave an empty line in between for clarity
        end

        def self.parse(body)
          return '' if body.blank?

          body_with_suggestions = body
            .scan(CODE_SUGGESTION_REGEX)
            .map do |header, multiline_from, multiline_to, inline_from, inline_to, footer|
              # NOTE: We're just interested in counting the existing lines as LLM doesn't
              #   seem to be able to reliably set this by itself.
              #   Also, since we have two optional matching pairs so either `multiline_from` and `multiline_to` or
              #   `inline_from` and `inline_to` would exist.
              line_offset_below = (multiline_from || inline_from).lines.count - 1

              # NOTE: Inline code suggestion needs to be wrapped in new lines to format it correctly.
              comment = inline_to.nil? ? multiline_to : "\n#{inline_to}\n"

              "#{header}```suggestion:-0+#{line_offset_below}#{comment}```#{footer}"
            end

          # NOTE: Return original body if the body doesn't have any expected suggestion format.
          return body unless body_with_suggestions.present?

          body_with_suggestions.join
        end
      end
    end
  end
end
