# frozen_string_literal: true

require 'spec_helper'

using RSpec::Parameterized::TableSyntax

RSpec.describe Ai::AmazonQ, feature_category: :ai_abstraction_layer do
  let(:application_settings) { ::Gitlab::CurrentSettings.current_application_settings }

  describe '#connected?' do
    where(:q_available, :q_ready, :result) do
      true  | true  | true
      true  | false | false
      false | true  | false
      false | false | false
    end

    with_them do
      before do
        Ai::Setting.instance.update!(amazon_q_ready: q_ready)
        allow(described_class).to receive(:feature_available?).and_return(q_available)
      end

      it 'returns the expected result' do
        expect(described_class.connected?).to be result
      end
    end
  end

  describe '#feature_available?' do
    where(:add_on_purchase, :amazon_q_license_available, :ff_enabled, :result) do
      nil                | true  | false | true
      :product_analytics | true  | false | true
      :gitlab_duo_pro    | true  | true  | true
      :duo_enterprise    | true  | true  | true
      :gitlab_duo_pro    | true  | false | false
      :duo_enterprise    | true  | false | false
      nil                | false | false | false
      nil                | false | true  | false
    end

    with_them do
      before do
        stub_licensed_features(amazon_q: amazon_q_license_available)
        stub_feature_flags(amazon_q_chat_and_code_suggestions: ff_enabled)

        create(:gitlab_subscription_add_on_purchase, add_on_purchase, namespace: nil) if add_on_purchase.present?
      end

      it 'returns the expected result' do
        expect(described_class.feature_available?).to eq(result)
      end
    end
  end

  describe '#enabled?' do
    context 'with args' do
      let(:namespace) { build(:project_namespace) }
      let(:user) { build(:user) }

      where(:feature_enabled, :connected, :result) do
        true     | true  | true
        true     | false | false
        false    | true  | false
        true     | false | false
      end

      with_them do
        before do
          allow(described_class).to receive_messages(
            feature_available?: feature_enabled,
            connected?: connected
          )
        end

        it { expect(described_class.enabled?).to eq(result) }
      end
    end
  end

  describe '#should_block_service_account?' do
    where(:availability, :expectation) do
      "default_on"  | false
      "default_off" | false
      "never_on"    | true
    end

    with_them do
      it { expect(described_class.should_block_service_account?(availability: availability)).to be(expectation) }
    end
  end

  describe '#ensure_service_account_blocked!' do
    let_it_be(:current_user) { create(:user, :admin) }
    let_it_be_with_reload(:service_account_normal) { create(:user, :service_account) }
    let_it_be_with_reload(:service_account_blocked) { create(:user, :service_account, :blocked) }
    let_it_be(:service_account_not_found) { Struct.new(:id).new(999999) }

    context 'with service_account set in application settings' do
      where(:service_account, :expected_service_class, :expected_status, :expected_message) do
        ref(:service_account_normal)     | ::Users::BlockService | true | nil
        ref(:service_account_blocked)    | nil | true | "Service account already blocked. Nothing to do."
      end

      with_them do
        before do
          Ai::Setting.instance.update!(amazon_q_service_account_user_id: service_account&.id)
        end

        it 'conditionally block the service account', :aggregate_failures do
          if expected_service_class
            expect_next_instance_of(expected_service_class, current_user) do |instance|
              expect(instance).to receive(:execute).with(service_account).and_call_original
            end
          end

          response = described_class.ensure_service_account_blocked!(current_user: current_user)

          expect(response.success?).to be(expected_status)
          expect(response.message).to be(expected_message)
        end
      end
    end

    context 'with service_account set as argument' do
      it 'conditionally blocks the given service account', :aggregate_failures do
        expect(service_account_normal.blocked?).to be(false)

        response = described_class.ensure_service_account_blocked!(
          current_user: current_user,
          service_account: service_account_normal
        )

        expect(response.success?).to be(true)
        expect(service_account_normal.blocked?).to be(true)
      end
    end
  end

  describe '#ensure_service_account_unblocked!' do
    let_it_be(:current_user) { create(:user, :admin) }
    let_it_be_with_reload(:service_account_normal) { create(:user, :service_account) }
    let_it_be_with_reload(:service_account_blocked) { create(:user, :service_account, :blocked) }

    context 'with service_account set in application settings' do
      where(:service_account, :expected_service_class, :expected_status, :expected_message) do
        ref(:service_account_normal)     | nil | true | "Service account already unblocked. Nothing to do."
        ref(:service_account_blocked)    | ::Users::UnblockService | true | nil
      end

      with_them do
        before do
          Ai::Setting.instance.update!(amazon_q_service_account_user_id: service_account&.id)
        end

        it 'conditionally block the service account', :aggregate_failures do
          if expected_service_class
            expect_next_instance_of(expected_service_class, current_user) do |instance|
              expect(instance).to receive(:execute).with(service_account).and_call_original
            end
          end

          response = described_class.ensure_service_account_unblocked!(current_user: current_user)

          expect(response.success?).to be(expected_status)
          expect(response.message).to be(expected_message)
        end
      end
    end

    context 'with service_account set as argument' do
      it 'conditionally blocks the given service account', :aggregate_failures do
        expect(service_account_blocked.blocked?).to be(true)

        response = described_class.ensure_service_account_unblocked!(
          current_user: current_user,
          service_account: service_account_blocked
        )

        expect(response.success?).to be(true)
        expect(service_account_blocked.blocked?).to be(false)
      end
    end
  end
end
