# frozen_string_literal: true

require 'spec_helper'

RSpec.describe Search::Zoekt::Index, feature_category: :global_search do
  let_it_be(:namespace) { create(:group) }
  let_it_be_with_reload(:zoekt_enabled_namespace) { create(:zoekt_enabled_namespace, namespace: namespace) }
  let_it_be(:zoekt_node) { create(:zoekt_node) }
  let_it_be(:zoekt_replica) { create(:zoekt_replica, zoekt_enabled_namespace: zoekt_enabled_namespace) }
  let_it_be_with_refind(:zoekt_index) do
    create(:zoekt_index, zoekt_enabled_namespace: zoekt_enabled_namespace, node: zoekt_node, replica: zoekt_replica,
      reserved_storage_bytes: 100.megabytes)
  end

  subject { zoekt_index }

  describe 'relations' do
    it { is_expected.to belong_to(:zoekt_enabled_namespace).inverse_of(:indices) }
    it { is_expected.to belong_to(:node).inverse_of(:indices) }
    it { is_expected.to belong_to(:replica).inverse_of(:indices) }
    it { is_expected.to have_many(:zoekt_repositories).inverse_of(:zoekt_index) }

    it 'restricts deletion when there are associated zoekt repositories' do
      project = create(:project, namespace_id: zoekt_index.namespace_id)
      repo = zoekt_index.zoekt_repositories.create!(project: project, state: :pending)

      expect(zoekt_index.zoekt_repositories).to match_array([repo])
      expect { zoekt_index.destroy! }.to raise_error ActiveRecord::InvalidForeignKey

      repo.destroy!

      expect { zoekt_index.destroy! }.not_to raise_error
    end
  end

  it { expect(described_class.new.reserved_storage_bytes).to eq 10.gigabytes }

  describe 'validations' do
    it 'validates that zoekt_enabled_namespace root_namespace_id matches namespace_id' do
      zoekt_index = described_class.new(zoekt_enabled_namespace: zoekt_enabled_namespace,
        node: zoekt_node, namespace_id: 0)
      expect(zoekt_index).to be_invalid
    end

    it 'allows a project_id_to value in metadata to be either an integer or nil' do
      expect(described_class.new(metadata: { 'project_id_from' => 123, 'project_id_to' => 456 })).to be_valid
      expect(described_class.new(metadata: { 'project_id_from' => 123, 'project_id_to' => nil })).to be_valid
    end
  end

  describe 'scopes' do
    let_it_be(:namespace_2) { create(:group) }
    let_it_be_with_reload(:zoekt_enabled_namespace_2) { create(:zoekt_enabled_namespace, namespace: namespace_2) }
    let_it_be(:node_2) { create(:zoekt_node) }
    let_it_be(:zoekt_index_2) do
      create(:zoekt_index, node: node_2, zoekt_enabled_namespace: zoekt_enabled_namespace_2)
    end

    before do
      create_list(:zoekt_repository, 5, zoekt_index: zoekt_index, size_bytes: 100.megabytes)
    end

    describe '.for_node' do
      subject { described_class.for_node(node_2) }

      it { is_expected.to contain_exactly(zoekt_index_2) }
    end

    describe '.for_root_namespace_id' do
      subject { described_class.for_root_namespace_id(namespace_2) }

      it { is_expected.to contain_exactly(zoekt_index_2) }

      context 'when there are orphaned indices' do
        before do
          zoekt_index_2.update!(zoekt_enabled_namespace: nil)
        end

        it { is_expected.to be_empty }
      end
    end

    describe '.for_root_namespace_id_with_search_enabled' do
      it 'correctly filters on the search field' do
        expect(described_class.for_root_namespace_id_with_search_enabled(namespace_2))
          .to contain_exactly(zoekt_index_2)

        zoekt_enabled_namespace_2.update!(search: false)

        expect(described_class.for_root_namespace_id_with_search_enabled(namespace_2))
          .to be_empty
      end
    end

    describe '.with_all_finished_repositories' do
      let_it_be(:idx) { create(:zoekt_index) } # It has some pending and some ready zoekt_repositories
      let_it_be(:idx2) { create(:zoekt_index) } # It has all ready zoekt_repositories
      let_it_be(:idx3) { create(:zoekt_index) } # It does not have zoekt_repositories
      let_it_be(:idx4) { create(:zoekt_index) } # It has all failed zoekt_repositories
      let_it_be(:idx5) { create(:zoekt_index) } # It has some failed and some ready zoekt_repositories
      let_it_be(:idx6) { create(:zoekt_index) } # It has some initializing and some pending zoekt_repositories
      let_it_be(:idx_project) { create(:project, namespace_id: idx.namespace_id) }
      let_it_be(:idx_project2) { create(:project, namespace_id: idx.namespace_id) }
      let_it_be(:idx2_project2) { create(:project, namespace_id: idx2.namespace_id) }
      let_it_be(:idx4_project) { create(:project, namespace_id: idx4.namespace_id) }
      let_it_be(:idx5_project) { create(:project, namespace_id: idx5.namespace_id) }
      let_it_be(:idx5_project2) { create(:project, namespace_id: idx5.namespace_id) }
      let_it_be(:idx6_project) { create(:project, namespace_id: idx6.namespace_id) }
      let_it_be(:idx6_project2) { create(:project, namespace_id: idx6.namespace_id) }

      before do
        idx.zoekt_repositories.create!(zoekt_index: idx, project: idx_project, state: :pending)
        idx.zoekt_repositories.create!(zoekt_index: idx, project: idx_project2, state: :ready)
        idx2.zoekt_repositories.create!(zoekt_index: idx2, project: idx2_project2, state: :ready)
        idx4.zoekt_repositories.create!(zoekt_index: idx2, project: idx4_project, state: :failed)
        idx5.zoekt_repositories.create!(zoekt_index: idx2, project: idx5_project, state: :failed)
        idx5.zoekt_repositories.create!(zoekt_index: idx2, project: idx5_project2, state: :ready)
        idx6.zoekt_repositories.create!(zoekt_index: idx6, project: idx6_project, state: :initializing)
        idx6.zoekt_repositories.create!(zoekt_index: idx6, project: idx6_project2, state: :pending)
      end

      it 'returns all the indices whose all zoekt_repositories are ready' do
        expect(described_class.with_all_finished_repositories).to include(idx2, idx3, idx4, idx5)
        expect(described_class.with_all_finished_repositories).not_to include(idx, idx6)
      end
    end

    describe '.ordered_by_used_storage_updated_at' do
      let_it_be(:zoekt_index_3) { create(:zoekt_index) }

      subject(:results) { described_class.ordered_by_used_storage_updated_at }

      it 'returns all indices in ascending order by used_storage_bytes_updated_at' do
        zoekt_index.update!(used_storage_bytes_updated_at: 10.minutes.ago)
        zoekt_index_2.update!(used_storage_bytes_updated_at: 5.hours.ago)
        zoekt_index_3.update!(used_storage_bytes_updated_at: 2.minutes.ago)
        expect(results.pluck(:id)).to eq([zoekt_index_2.id, zoekt_index.id, zoekt_index_3.id])
      end
    end

    describe '.with_stale_used_storage_bytes_updated_at' do
      let_it_be(:time) { Time.zone.now }
      let_it_be(:idx) { create(:zoekt_index) }
      let_it_be(:idx_2) { create(:zoekt_index, :stale_used_storage_bytes_updated_at) }
      let_it_be(:idx_3) do
        create(:zoekt_index, used_storage_bytes_updated_at: time, last_indexed_at: time - 5.seconds)
      end

      subject(:results) { described_class.with_stale_used_storage_bytes_updated_at }

      it 'returns all the indices whose used_storage_bytes_updated_at is less than last_indexed_at' do
        expect(results).to include idx, idx_2
        expect(results).not_to include idx_3
      end
    end

    describe '.pre_ready' do
      let_it_be(:in_progress) { create(:zoekt_index, state: :in_progress) }
      let_it_be(:initializing) { create(:zoekt_index, state: :initializing) }
      let_it_be(:ready) { create(:zoekt_index, state: :ready) }
      let_it_be(:reallocating) { create(:zoekt_index, state: :reallocating) }
      let_it_be(:pending_deletion) { create(:zoekt_index, state: :pending_deletion) }

      it 'returns correct indices' do
        expect(described_class.pre_ready).to contain_exactly(zoekt_index, zoekt_index_2, in_progress, initializing)
      end
    end

    describe '.searchable' do
      let_it_be(:zoekt_index_ready) do
        create(:zoekt_index, node: zoekt_node, zoekt_enabled_namespace: zoekt_enabled_namespace_2, state: :ready)
      end

      it 'returns correct indices' do
        expect(described_class.searchable).to contain_exactly(zoekt_index_ready)
      end
    end

    describe '.preload_zoekt_enabled_namespace_and_namespace' do
      it 'preloads the project and avoids N+1 queries' do
        index = described_class.preload_zoekt_enabled_namespace_and_namespace.first
        recorder = ActiveRecord::QueryRecorder.new { index.zoekt_enabled_namespace.namespace }
        expect(recorder.count).to be_zero
      end
    end

    describe '.preload_node' do
      it 'preloads the node and avoids N+1 queries' do
        index = described_class.preload_node.first
        recorder = ActiveRecord::QueryRecorder.new { index.node }
        expect(recorder.count).to be_zero
      end
    end

    describe '.negative_reserved_storage_bytes' do
      let_it_be(:negative_reserved_storage_bytes_index) { create(:zoekt_index, :negative_reserved_storage_bytes) }

      it 'returns indices only with negative reserved_storage_bytes' do
        results = described_class.negative_reserved_storage_bytes
        expect(results.all? { |idx| idx.reserved_storage_bytes < 0 }).to be true
        expect(results).to include(negative_reserved_storage_bytes_index)
      end
    end

    describe '.should_be_marked_as_orphaned' do
      let_it_be(:idx) { create(:zoekt_index) }
      let_it_be(:idx_missing_replica) { create(:zoekt_index) }
      let_it_be(:idx_missing_enabled_namespace) { create(:zoekt_index) }
      let_it_be(:idx_already_marked_as_orphaned) { create(:zoekt_index) }
      let_it_be(:zoekt_replica) { create(:zoekt_replica) }

      it 'returns indices that are missing either an enabled namespace or a replica' do
        idx_missing_replica.replica.destroy!
        idx_missing_enabled_namespace.zoekt_enabled_namespace.destroy!
        idx_already_marked_as_orphaned.replica.destroy!
        idx_already_marked_as_orphaned.orphaned!

        expect(described_class.should_be_marked_as_orphaned).to match_array([idx_missing_replica,
          idx_missing_enabled_namespace])
      end
    end

    describe '.should_be_deleted' do
      let_it_be(:idx) { create(:zoekt_index) }
      let_it_be(:idx_orphaned) { create(:zoekt_index, state: :orphaned) }
      let_it_be(:idx_pending_deletion) { create(:zoekt_index, state: :pending_deletion) }

      it 'returns indices that are marked as either orphaned or pending_deletion' do
        expect(described_class.should_be_deleted).to match_array([idx_orphaned, idx_pending_deletion])
      end
    end

    describe '.should_be_pending_eviction' do
      let_it_be(:idx_healthy) { create(:zoekt_index, :healthy) }
      let_it_be(:idx_critical_watermark_exceeded) { create(:zoekt_index, :critical_watermark_exceeded) }
      let_it_be(:idx_pending_eviction) do
        create(:zoekt_index, :critical_watermark_exceeded, state: :pending_eviction)
      end

      let_it_be(:idx_evicted) do
        create(:zoekt_index, :critical_watermark_exceeded, state: :evicted)
      end

      let_it_be(:idx_orphaned) do
        create(:zoekt_index, :critical_watermark_exceeded, state: :orphaned)
      end

      let_it_be(:idx_pending_deletion) do
        create(:zoekt_index, :critical_watermark_exceeded, state: :pending_deletion)
      end

      it 'returns indices that are idx_critical_watermark_exceeded that contain zoekt_replica_id' do
        expect(described_class.should_be_pending_eviction).to match_array([idx_critical_watermark_exceeded])
      end
    end

    describe '.with_mismatched_watermark_levels' do
      let(:ideal_percent) { Search::Zoekt::Index::STORAGE_IDEAL_PERCENT_USED }
      let(:low_watermark) { Search::Zoekt::Index::STORAGE_LOW_WATERMARK }
      let(:high_watermark) { Search::Zoekt::Index::STORAGE_HIGH_WATERMARK }
      let(:critical_watermark) { Search::Zoekt::Index::STORAGE_CRITICAL_WATERMARK }
      let(:mismatched_indices) { described_class.with_mismatched_watermark_levels }

      before do
        # Clear existing records
        Search::Zoekt::Repository.delete_all
        described_class.delete_all
      end

      it 'returns indices where watermark_level is mismatched (healthy)' do
        # Setup a healthy record but with incorrect watermark_level
        idx = create(
          :zoekt_index,
          used_storage_bytes: 40,
          reserved_storage_bytes: 100
        )

        # Skip active record callback: set_watermark_level
        idx.update!(watermark_level: :low_watermark_exceeded) # Incorrect level

        expect(mismatched_indices.count).to eq(1)
        expect(mismatched_indices.first.watermark_level).to eq('low_watermark_exceeded')
      end

      it 'returns no indices when all watermark_levels are correct' do
        # Setup record with correct watermark level
        create(
          :zoekt_index,
          used_storage_bytes: 40,
          reserved_storage_bytes: 100,
          watermark_level: :healthy
        )

        expect(mismatched_indices).to be_empty
      end

      it 'detects overprovisioned mismatches' do
        # Setup an overprovisioned record with incorrect watermark_level
        idx = create(
          :zoekt_index,
          used_storage_bytes: 10,
          reserved_storage_bytes: 100
        )

        # Skip active record callback: set_watermark_level
        idx.update!(watermark_level: :healthy) # Incorrect level

        expect(mismatched_indices.count).to eq(1)
        expect(mismatched_indices.first.watermark_level).to eq('healthy')
      end

      it 'handles edge cases at the exact boundary' do
        # Setup a record exactly at the STORAGE_LOW_WATERMARK
        idx = create(
          :zoekt_index,
          used_storage_bytes: (low_watermark * 100).to_i,
          reserved_storage_bytes: 100
        )

        # Skip active record callback
        idx.update!(
          watermark_level: :healthy # Incorrect level
        )

        expect(mismatched_indices.count).to eq(1)
        expect(mismatched_indices.first.watermark_level).to eq('healthy')
      end

      it 'handles division by zero gracefully' do
        # Setup a record with zero reserved_storage_bytes
        create(
          :zoekt_index,
          used_storage_bytes: 50,
          reserved_storage_bytes: 0,
          watermark_level: :critical_watermark_exceeded
        )

        expect { mismatched_indices }.not_to raise_error
      end

      it 'returns indices where watermark_level is mismatched (critical)' do
        # Setup a record that should be critical but has incorrect watermark_level
        idx = create(
          :zoekt_index,
          used_storage_bytes: (critical_watermark * 100) + 1,
          reserved_storage_bytes: 100
        )

        idx.update!(watermark_level: :high_watermark_exceeded) # Incorrect level

        expect(mismatched_indices.count).to eq(1)
        expect(mismatched_indices.first.watermark_level).to eq('high_watermark_exceeded')
      end

      it 'correctly identifies critical watermark level' do
        # Setup a record with correct critical watermark level
        create(
          :zoekt_index,
          used_storage_bytes: (critical_watermark * 100) + 1,
          reserved_storage_bytes: 100,
          watermark_level: :critical_watermark_exceeded
        )

        expect(mismatched_indices).to be_empty
      end
    end
  end

  describe '#refresh_reserved_storage_bytes' do
    let(:zoekt_node) { create(:zoekt_node, total_bytes: 1000, used_bytes: 300, indexed_bytes: 200) }
    let(:idx) do
      create(:zoekt_index, used_storage_bytes: 90, reserved_storage_bytes: 100, node: zoekt_node)
    end

    before do
      zoekt_node.save! # This triggers update_usable_storage_bytes to set usable_storage_bytes to 900
      # Make sure we properly stub both indices and reserved_storage_bytes
      allow(zoekt_node).to receive_messages(indices: [], reserved_storage_bytes: 0)
    end

    it 'updates reserved_storage_bytes with the ideal storage based on used bytes' do
      ideal_reserved_storage = idx.used_storage_bytes / described_class::STORAGE_IDEAL_PERCENT_USED

      expect do
        idx.refresh_reserved_storage_bytes
      end.to change {
        idx.reserved_storage_bytes
      }.from(100).to(ideal_reserved_storage)
    end

    context 'when the node has limited storage' do
      let(:zoekt_node) { create(:zoekt_node, total_bytes: 300, used_bytes: 198, indexed_bytes: 0) }
      let(:idx) do
        create(:zoekt_index, used_storage_bytes: 90, reserved_storage_bytes: 100, node: zoekt_node)
      end

      let(:other_index) { create(:zoekt_index, node: zoekt_node, reserved_storage_bytes: 100) }

      before do
        zoekt_node.save! # This triggers update_usable_storage_bytes to set usable_storage_bytes to 102
        allow(zoekt_node).to receive_messages(indices: [other_index], reserved_storage_bytes: 100)
      end

      it 'increases the reserved storage only up to available node storage' do
        # With the new formula:
        # unclaimed_storage_bytes = usable_storage_bytes - reserved_storage_bytes
        # unclaimed_storage_bytes = 102 - 100 = 2
        #
        # Available space for this index = unclaimed_storage_bytes + this index's current reserved_storage_bytes
        # Available space = 2 + 100 = 102

        expect do
          idx.refresh_reserved_storage_bytes
        end.to change {
          idx.reserved_storage_bytes
        }.from(100).to(102)
      end
    end

    context 'when the node does not have any more storage' do
      let(:zoekt_node) { create(:zoekt_node, total_bytes: 300, used_bytes: 300, indexed_bytes: 0) }
      let(:idx) do
        create(:zoekt_index, used_storage_bytes: 90, reserved_storage_bytes: 100, node: zoekt_node)
      end

      before do
        zoekt_node.save! # This triggers update_usable_storage_bytes to set usable_storage_bytes to 0

        # Set up the node to indicate it has no more available storage
        # This means: usable_storage_bytes <= reserved_storage_bytes
        # which gives unclaimed_storage_bytes <= 0
        allow(zoekt_node).to receive_messages(usable_storage_bytes: 0, indices: [], reserved_storage_bytes: 0)

        # Important: unclaimed_storage_bytes + current reserved space is now 0,
        # so there's no room to grow beyond current reserved space

        # Additionally, stub the index's ideal size determination to ensure
        # it doesn't trigger the change
        allow(idx).to receive(:ideal_reserved_storage_bytes).and_return(200)
      end

      it 'does not change reserved_storage_bytes' do
        expect do
          idx.refresh_reserved_storage_bytes
        end.not_to change { idx.reserved_storage_bytes }
      end
    end

    context 'when used_storage_bytes is 0' do
      let(:idx) { create(:zoekt_index, used_storage_bytes: 0) }

      it 'sets reserved_storage_bytes to DEFAULT_RESERVED_STORAGE_BYTES' do
        expect do
          idx.refresh_reserved_storage_bytes
        end.to change {
          idx.reserved_storage_bytes
        }.to(described_class::DEFAULT_RESERVED_STORAGE_BYTES)
      end

      it 'does not perform division calculation' do
        expect(idx).not_to receive(:storage_percent_used)
        idx.refresh_reserved_storage_bytes
      end

      it 'does not check node unclaimed storage' do
        expect(idx.node).not_to receive(:unclaimed_storage_bytes)
        idx.refresh_reserved_storage_bytes
      end
    end
  end

  describe '#free_storage_bytes' do
    it 'is difference between reserved bytes and used bytes' do
      allow(zoekt_index).to receive_messages(reserved_storage_bytes: 100, used_storage_bytes: 1)
      expect(zoekt_index.free_storage_bytes).to eq(99)
    end
  end

  describe '#update_storage_bytes!' do
    using RSpec::Parameterized::TableSyntax

    before do
      allow(zoekt_index).to receive(:refresh_used_storage_bytes)
      allow(zoekt_index).to receive(:refresh_reserved_storage_bytes)
      allow(zoekt_index).to receive(:save!)
      allow(zoekt_index).to receive_messages(ready?: ready, overprovisioned?: overprovisioned,
        high_watermark_exceeded?: high_watermark_exceeded)
    end

    where(:ready, :overprovisioned, :high_watermark_exceeded, :should_update_reserved_storage) do
      true  | true  | false | true
      true  | false | true  | true
      true  | false | false | false
      false | true  | false | false
      false | false | true  | true
      false | false | false | false
    end

    with_them do
      it 'calls the correct methods' do
        expect(zoekt_index).to receive(:refresh_used_storage_bytes).once
        expect(zoekt_index).to receive(:save!).once

        if should_update_reserved_storage
          expect(zoekt_index).to receive(:refresh_reserved_storage_bytes).once
        else
          expect(zoekt_index).not_to receive(:refresh_reserved_storage_bytes)
        end

        zoekt_index.update_storage_bytes!
      end
    end
  end

  describe '#refresh_used_storage_bytes', :freeze_time do
    let_it_be(:initial_time) { 10.minutes.ago }
    let_it_be(:current_time) { Time.zone.now }
    let_it_be(:zoekt_index) do
      build(:zoekt_index, used_storage_bytes_updated_at: initial_time)
    end

    context 'when there are repositories with size_bytes' do
      let(:repository_size) { 100.megabytes }
      let(:repositories_association) { instance_double(ActiveRecord::Associations::CollectionProxy) }

      before do
        allow(zoekt_index).to receive(:zoekt_repositories).and_return(repositories_association)
        allow(repositories_association).to receive(:sum).with(:size_bytes).and_return(repository_size * 3)
      end

      it 'updates used_storage_bytes to the sum of repository sizes' do
        expect { zoekt_index.refresh_used_storage_bytes }.to change {
          zoekt_index.used_storage_bytes
        }.to(repository_size * 3)
      end

      it 'updates used_storage_bytes_updated_at to current time' do
        zoekt_index.refresh_used_storage_bytes
        expect(zoekt_index.used_storage_bytes_updated_at).to be_like_time(current_time)
      end
    end

    context 'when there are no repositories or all repositories have zero size' do
      let(:repositories_association) { instance_double(ActiveRecord::Associations::CollectionProxy) }

      before do
        allow(zoekt_index).to receive(:zoekt_repositories).and_return(repositories_association)
        allow(repositories_association).to receive(:sum).with(:size_bytes).and_return(0)
      end

      it 'sets used_storage_bytes to DEFAULT_USED_STORAGE_BYTES' do
        expect { zoekt_index.refresh_used_storage_bytes }.to change {
          zoekt_index.used_storage_bytes
        }.to(described_class::DEFAULT_USED_STORAGE_BYTES)
      end

      it 'updates used_storage_bytes_updated_at to current time' do
        zoekt_index.refresh_used_storage_bytes
        expect(zoekt_index.used_storage_bytes_updated_at).to be_like_time(current_time)
      end
    end
  end

  describe '#should_be_deleted?' do
    it 'returns true if the index state is orphaned or pending_deletion' do
      expect(zoekt_index).not_to be_should_be_deleted

      zoekt_index.state = :orphaned
      expect(zoekt_index).to be_should_be_deleted

      zoekt_index.state = :pending_deletion
      expect(zoekt_index).to be_should_be_deleted

      zoekt_index.state = :ready
      expect(zoekt_index).not_to be_should_be_deleted
    end
  end

  describe '#find_or_create_repository_by_project!' do
    let_it_be(:zoekt_index) { create(:zoekt_index) }
    let_it_be(:project) { create(:project) }

    context 'when find_or_create_by! raises an error' do
      before do
        allow(zoekt_index).to receive_message_chain(:zoekt_repositories, :find_or_create_by!).and_raise(StandardError)
      end

      it 'raises the error' do
        expect do
          zoekt_index.find_or_create_repository_by_project!(project.id, project)
        end.to raise_error(StandardError).and not_change { Search::Zoekt::Repository.count }
      end
    end

    context 'when zoekt_repository exists with the given params' do
      before do
        create(:zoekt_repository, project: project, zoekt_index: zoekt_index)
      end

      it 'returns the existing record' do
        result = nil
        expect do
          result = zoekt_index.find_or_create_repository_by_project!(project.id, project)
        end.not_to change { zoekt_index.zoekt_repositories.count }
        expect(result.project_identifier).to eq project.id
      end
    end

    context 'when zoekt_repository does not exists with the given params' do
      it 'creates and return the new record' do
        result = nil
        expect do
          result = zoekt_index.find_or_create_repository_by_project!(project.id, project)
        end.to change { zoekt_index.zoekt_repositories.count }.by(1)
        expect(result.project_identifier).to eq project.id
      end
    end
  end
end
