# frozen_string_literal: true

require 'spec_helper'

RSpec.describe Security::TokenRevocationService, '#execute', feature_category: :security_policy_management do
  let_it_be(:revocation_token_types_url) { 'https://myhost.com/api/v1/token_types' }
  let_it_be(:token_revocation_url) { 'https://myhost.com/api/v1/revoke' }
  let_it_be(:project) { build(:project) }

  let_it_be(:revocable_keys) do
    [{
      type: 'aws_key_id',
      token: 'AKIASOMEAWSACCESSKEY',
      location: 'https://mywebsite.com/some-repo/blob/abcdefghijklmnop/compromisedfile.java'
     },
     {
        type: 'aws_secret',
        token: 'some_aws_secret_key_some_aws_secret_key_',
        location: 'https://mywebsite.com/some-repo/blob/abcdefghijklmnop/compromisedfile.java'
     },
     {
        type: 'aws_secret',
        token: 'another_aws_secret_key_another_secret_key',
        location: 'https://mywebsite.com/some-repo/blob/abcdefghijklmnop/compromisedfile.java'
     }]
  end

  let_it_be(:revocable_external_token_types) do
    { types: %w[aws_key_id aws_secret gcp_key_id gcp_secret] }
  end

  subject(:token_revocation_service) { described_class.new(revocable_keys:, project:).execute }

  before do
    stub_application_setting(secret_detection_revocation_token_types_url: revocation_token_types_url)
    stub_application_setting(secret_detection_token_revocation_token: 'token1')
    stub_application_setting(secret_detection_token_revocation_url: token_revocation_url)
  end

  context 'when revoking a glpat token' do
    let_it_be(:glpat_token) { create(:personal_access_token) }
    let_it_be(:vulnerability) { create(:vulnerability, project: project) }

    let_it_be(:revocable_keys) do
      [
        {
          type: 'gitleaks_rule_id_gitlab_personal_access_token',
          token: glpat_token.token,
          location: 'https://example.com/some-repo/blob/abcdefghijklmnop/compromisedfile1.java#L21',
          vulnerability: vulnerability
        },
        {
          type: 'gitleaks_rule_id_gitlab_personal_access_token',
          token: glpat_token.token,
          location: 'https://example.com/some-repo/blob/abcdefghijklmnop/compromisedfile1.java#L41',
          vulnerability: vulnerability
        }
      ]
    end

    it_behaves_like 'internal event tracking' do
      let(:event) { 'revoke_leaked_token_after_vulnerability_report_is_ingested' }
      let(:category) { described_class.name }
      let(:additional_properties) { { label: 'gitleaks_rule_id_gitlab_personal_access_token' } }
    end

    it 'returns success' do
      expect(PersonalAccessTokens::RevokeService).to receive(:new).once.and_call_original

      audit_context = {
        name: 'personal_access_token_revoked',
        author: Users::Internal.security_bot,
        scope: Users::Internal.security_bot,
        target: glpat_token.user,
        message: "Revoked personal access token with id #{glpat_token.id}",
        additional_details: {
          revocation_source: :secret_detection
        }
      }

      expect(::Gitlab::Audit::Auditor).to receive(:audit).with(audit_context).and_call_original

      expect(SystemNoteService)
        .to receive(:change_vulnerability_state)
        .with(
          vulnerability,
          Users::Internal.security_bot,
          s_("TokenRevocation|This personal access token has been automatically revoked on detection. " \
             "Consider investigating and rotating before marking this vulnerability as resolved.")
        )

      expect(subject[:status]).to be(:success)
    end

    context 'when vulnerability is missing' do
      before do
        revocable_keys.each do |key|
          key.delete(:vulnerability)
        end
      end

      it 'does not call `SystemNoteService`' do
        expect(SystemNoteService).not_to receive(:change_vulnerability_state)

        subject
      end

      it_behaves_like 'internal event tracking' do
        let(:event) { 'revoke_leaked_token_after_vulnerability_report_is_ingested' }
        let(:category) { described_class.name }
        let(:additional_properties) { { label: 'gitleaks_rule_id_gitlab_personal_access_token' } }
      end
    end
  end

  context 'when revocation token API returns a response with failure' do
    before do
      stub_application_setting(secret_detection_token_revocation_enabled: true)
      stub_revoke_token_api_with_failure
      stub_revocation_token_types_api_with_success
    end

    it 'returns error' do
      expect(subject[:status]).to be(:error)
      expect(subject[:message]).to eql('Failed to revoke tokens')
    end

    it 'does not create internal tracking events' do
      expect { subject }.not_to trigger_internal_events('revoke_leaked_token_after_vulnerability_report_is_ingested')
    end
  end

  context 'when revocation token types API returns empty list of types' do
    before do
      stub_application_setting(secret_detection_token_revocation_enabled: true)
      stub_invalid_token_types_api_with_success
    end

    specify { expect(subject).to eql({ status: :success }) }
  end

  context 'when external revocation service is disabled' do
    specify { expect(subject).to eql({ status: :success }) }
  end

  context 'when external revocation service is enabled' do
    before do
      stub_application_setting(secret_detection_token_revocation_enabled: true)
      stub_revoke_token_api_with_success
    end

    context 'with a list of valid token types' do
      before do
        stub_revocation_token_types_api_with_success
      end

      context 'when there is a list of tokens to be revoked' do
        describe 'status is success' do
          specify { expect(described_class.new(revocable_keys:, project:).execute[:status]).to be(:success) }
        end

        # rubocop:disable Layout/LineLength -- metric names are very long
        it 'creates internal tracking event' do
          expect { described_class.new(revocable_keys:, project:).execute }.to trigger_internal_events(
            'revoke_leaked_token_after_vulnerability_report_is_ingested').with(
              project: project, namespace: project.namespace, additional_properties: { label: 'aws_key_id' }
            ).and trigger_internal_events('revoke_leaked_token_after_vulnerability_report_is_ingested').with(
              project: project, namespace: project.namespace, additional_properties: { label: 'aws_secret' }
            ).twice.and increment_usage_metrics(
              'redis_hll_counters.count_distinct_namespace_id_from_revoke_leaked_token_after_vulnerability_report_is_ingested_monthly',
              'redis_hll_counters.count_distinct_project_id_from_revoke_leaked_token_after_vulnerability_report_is_ingested_monthly',
              'redis_hll_counters.count_distinct_namespace_id_from_revoke_leaked_token_after_vulnerability_report_is_ingested_weekly',
              'redis_hll_counters.count_distinct_project_id_from_revoke_leaked_token_after_vulnerability_report_is_ingested_weekly'
            ).by(1).and increment_usage_metrics(
              'redis_hll_counters.count_distinct_label_from_revoke_leaked_token_after_vulnerability_report_is_ingested_monthly',
              'redis_hll_counters.count_distinct_label_from_revoke_leaked_token_after_vulnerability_report_is_ingested_weekly'
            ).by(2)
        end
        # rubocop:enable Layout/LineLength
      end

      context 'when token_revocation_url is missing' do
        before do
          allow_next_instance_of(described_class) do |token_revocation_service|
            allow(token_revocation_service).to receive(:token_revocation_url) { nil }
          end
        end

        specify { expect(subject).to eql({ message: 'Missing revocation token data', status: :error }) }

        it 'does not create internal tracking events' do
          expect { subject }.not_to trigger_internal_events(
            'revoke_leaked_token_after_vulnerability_report_is_ingested'
          )
        end
      end

      context 'when token_types_url is missing' do
        before do
          allow_next_instance_of(described_class) do |token_revocation_service|
            allow(token_revocation_service).to receive(:token_types_url) { nil }
          end
        end

        specify { expect(subject).to eql({ message: 'Missing revocation token data', status: :error }) }

        it 'does not create internal tracking events' do
          expect { subject }.not_to trigger_internal_events(
            'revoke_leaked_token_after_vulnerability_report_is_ingested'
          )
        end
      end

      context 'when revocation_api_token is missing' do
        before do
          allow_next_instance_of(described_class) do |token_revocation_service|
            allow(token_revocation_service).to receive(:revocation_api_token) { nil }
          end
        end

        specify { expect(subject).to eql({ message: 'Missing revocation token data', status: :error }) }

        it 'does not create internal tracking events' do
          expect { subject }.not_to trigger_internal_events(
            'revoke_leaked_token_after_vulnerability_report_is_ingested'
          )
        end
      end

      context 'when there is no token to be revoked' do
        let_it_be(:revocable_external_token_types) do
          { types: %w[] }
        end

        specify { expect(subject).to eql({ status: :success }) }

        it 'does not create internal tracking events' do
          expect { subject }.not_to trigger_internal_events(
            'revoke_leaked_token_after_vulnerability_report_is_ingested'
          )
        end
      end
    end

    context 'when revocation token types API returns an unsuccessful response' do
      before do
        stub_revocation_token_types_api_with_failure
      end

      specify { expect(subject).to eql({ message: 'Failed to get revocation token types', status: :error }) }

      it 'does not create internal tracking events' do
        expect { subject }.not_to trigger_internal_events(
          'revoke_leaked_token_after_vulnerability_report_is_ingested'
        )
      end
    end
  end

  def stub_revoke_token_api_with_success
    stub_request(:post, token_revocation_url)
      .with(body: revocable_keys.to_json)
      .to_return(
        status: 200,
        headers: { 'Content-Type' => 'application/json' },
        body: {}.to_json
      )
  end

  def stub_revoke_token_api_with_failure
    stub_request(:post, token_revocation_url)
      .with(body: revocable_keys.to_json)
      .to_return(
        status: 400,
        headers: { 'Content-Type' => 'application/json' },
        body: {}.to_json
      )
  end

  def stub_revocation_token_types_api_with_success
    stub_request(:get, revocation_token_types_url)
      .to_return(
        status: 200,
        headers: { 'Content-Type' => 'application/json' },
        body: revocable_external_token_types.to_json
      )
  end

  def stub_invalid_token_types_api_with_success
    stub_request(:get, revocation_token_types_url)
      .to_return(
        status: 200,
        headers: { 'Content-Type' => 'application/json' },
        body: {}.to_json
      )
  end

  def stub_revocation_token_types_api_with_failure
    stub_request(:get, revocation_token_types_url)
      .to_return(
        status: 400,
        headers: { 'Content-Type' => 'application/json' },
        body: {}.to_json
      )
  end
end
