# frozen_string_literal: true

require 'spec_helper'

RSpec.describe SyncSeatLinkRequestWorker, type: :worker, feature_category: :plan_provisioning do
  describe '#perform' do
    subject(:sync_seat_link) do
      described_class.new.perform('2020-01-01T01:20:12+02:00', '123', 5, 4)
    end

    let(:subscription_portal_url) { ::Gitlab::Routing.url_helpers.subscription_portal_url }
    let(:seat_link_url) { [subscription_portal_url, '/api/v1/seat_links'].join }
    let(:body) { { success: true } }

    let_it_be(:organization) { create(:organization) }

    before do
      stub_request(:post, seat_link_url).to_return_json(
        status: 200,
        body: body.to_json
      )
    end

    it 'makes an HTTP POST request with passed params' do
      allow(Gitlab::CurrentSettings).to receive(:uuid).and_return('one-two-three')

      sync_seat_link

      expect(WebMock).to have_requested(:post, seat_link_url).with(
        body: {
          gitlab_version: Gitlab::VERSION,
          timestamp: '2020-01-01T01:20:12+02:00',
          license_key: '123',
          max_historical_user_count: 5,
          billable_users_count: 4,
          hostname: Gitlab.config.gitlab.host,
          instance_id: 'one-two-three',
          add_on_metrics: []
        }.to_json
      )
    end

    context 'when response contains a license' do
      let(:license_key) { build(:gitlab_license, :cloud).export }
      let(:body) { { success: true, license: license_key } }

      shared_examples 'clearing license cache' do
        it 'resets the current license cache', :request_store do
          # called twice because the current license cache is reset before checking the up to date current license
          # within this class and then again with the `after_commit :reset_current` when creating the new license
          expect(License).to receive(:reset_current).twice.and_call_original

          sync_seat_link
        end
      end

      shared_examples 'successful license creation' do
        it 'persists the new license' do
          freeze_time do
            expect { sync_seat_link }.to change(License, :count).by(1)
            expect(License.current).to have_attributes(
              data: license_key,
              cloud: true,
              last_synced_at: Time.current
            )
          end
        end
      end

      context 'when there is no previous license' do
        before do
          License.delete_all
        end

        it_behaves_like 'clearing license cache'
        it_behaves_like 'successful license creation'
      end

      context 'when there is a previous license' do
        before do
          License.current # set up license cache
        end

        context 'when it is a cloud license' do
          context 'when the current license key does not match the one returned from sync' do
            let!(:current_license) { create(:license, cloud: true, last_synced_at: 1.day.ago) }

            it_behaves_like 'clearing license cache'

            it 'creates a new license', :freeze_time do
              expect { sync_seat_link }.to change(License.cloud, :count).by(1)

              new_current_license = License.current
              expect(new_current_license).not_to eq(current_license.id)
              expect(new_current_license).to have_attributes(
                data: license_key,
                cloud: true,
                last_synced_at: Time.current
              )
            end
          end

          context 'when the current license key matches the one returned from sync' do
            let!(:current_license) { create(:license, cloud: true, data: license_key, last_synced_at: 1.day.ago) }

            it_behaves_like 'clearing license cache'

            it 'reuses the current license and updates the last_synced_at', :request_store, :freeze_time do
              expect { sync_seat_link }.not_to change(License.cloud, :count)

              expect(License.current).to have_attributes(
                id: current_license.id,
                data: license_key,
                cloud: true,
                last_synced_at: Time.current
              )
            end
          end

          context 'when persisting fails' do
            let(:license_key) { 'invalid-key' }
            let!(:current_license) { License.current }

            it 'resets the current license cache', :request_store do
              allow(Gitlab::ErrorTracking).to receive(:track_and_raise_for_dev_exception).and_call_original

              # called only once because no new license is created and the `after_commit :reset_current` isn't executed
              expect(License).to receive(:reset_current).once.and_call_original

              expect { sync_seat_link }.to raise_error ActiveRecord::RecordInvalid
            end

            it 'does not delete the current license and logs error' do
              expect(Gitlab::ErrorTracking).to receive(:track_and_raise_for_dev_exception).and_call_original

              expect { sync_seat_link }.to raise_error ActiveRecord::RecordInvalid

              expect(License).to exist(current_license.id)
            end
          end
        end

        context 'when it is not a cloud license' do
          let!(:current_license) { create(:license) }

          it_behaves_like 'clearing license cache'
          it_behaves_like 'successful license creation'
        end
      end
    end

    context 'when response contains reconciliation dates' do
      let(:body) { { success: true, next_reconciliation_date: today.to_s, display_alert_from: (today - 7.days).to_s } }
      let(:today) { Date.current }

      it 'creates reconciliation record with correct attributes' do
        sync_seat_link
        upcoming_reconciliation = GitlabSubscriptions::UpcomingReconciliation.next

        expect(upcoming_reconciliation.next_reconciliation_date).to eq(today)
        expect(upcoming_reconciliation.display_alert_from).to eq(today - 7.days)
        expect(upcoming_reconciliation.organization_id).to eq(organization.id)
      end

      context 'when an upcoming_reconciliation already exists' do
        it 'updates the upcoming_reconciliation' do
          upcoming_reconciliation = create(:upcoming_reconciliation, :self_managed, next_reconciliation_date: today + 2.days, display_alert_from: today + 1.day)

          sync_seat_link

          upcoming_reconciliation.reload

          expect(upcoming_reconciliation.next_reconciliation_date).to eq(today)
          expect(upcoming_reconciliation.display_alert_from).to eq(today - 7.days)
        end
      end
    end

    context 'when response contains future subscription information' do
      let(:future_subscriptions) { [{ 'foo' => 'bar' }] }
      let(:body) { { success: true, future_subscriptions: future_subscriptions } }
      let(:today) { Date.current }

      context 'when future subscription information is present in the response' do
        context 'and no future subscriptions are saved in the current settings' do
          it 'persists future subscription information' do
            expect { sync_seat_link }.to change { Gitlab::CurrentSettings.current_application_settings.future_subscriptions }.from([]).to(future_subscriptions)
          end
        end

        context 'and future subscriptions are saved in the current settings' do
          before do
            Gitlab::CurrentSettings.current_application_settings.update!(future_subscriptions: [{}])
          end

          it 'replaces future subscription information' do
            expect { sync_seat_link }.to change { Gitlab::CurrentSettings.current_application_settings.future_subscriptions }.from([{}]).to(future_subscriptions)
          end
        end
      end

      context 'when future subscription information is not present in the response' do
        let(:future_subscriptions) { [] }

        context 'and no future subscriptions are saved in the current settings' do
          it 'does not change the settings' do
            expect { sync_seat_link }.not_to change { Gitlab::CurrentSettings.current_application_settings.future_subscriptions }.from(future_subscriptions)
          end
        end

        context 'and future subscription are saved in the current settings' do
          before do
            Gitlab::CurrentSettings.current_application_settings.update!(future_subscriptions: [{}])
          end

          it 'clears future subscription information' do
            expect { sync_seat_link }.to change { Gitlab::CurrentSettings.current_application_settings.future_subscriptions }.from([{}]).to(future_subscriptions)
          end
        end
      end

      context 'when saving fails' do
        it 'logs error' do
          allow(Gitlab::CurrentSettings.current_application_settings).to receive(:save!).and_raise('saving fails')

          expect(Gitlab::ErrorTracking).to receive(:track_and_raise_for_dev_exception)
          expect { sync_seat_link }.not_to raise_error
        end
      end
    end

    context 'when new license does not contain a code suggestions add-on purchase' do
      it_behaves_like 'call runner to handle the provision of add-ons'
    end

    context 'when new license contains a code suggestions add-on purchase' do
      let(:license_key) do
        build(
          :gitlab_license,
          :cloud,
          restrictions: { code_suggestions_seat_count: 1, subscription_name: 'A-S00000001' }
        ).export
      end

      let(:body) { { success: true, license: license_key } }

      it_behaves_like 'call runner to handle the provision of add-ons'
    end

    context 'when the response does not contain reconciliation dates' do
      let(:body) do
        {
          success: true,
          next_reconciliation_date: nil,
          display_alert_from: nil
        }
      end

      it 'destroys the existing upcoming reconciliation record for the instance' do
        create(:upcoming_reconciliation, :self_managed)

        expect { sync_seat_link }
          .to change(GitlabSubscriptions::UpcomingReconciliation, :count)
          .by(-1)
      end

      it 'does not change anything when there is no existing record' do
        expect { sync_seat_link }.not_to change(GitlabSubscriptions::UpcomingReconciliation, :count)
      end
    end

    context 'when refresh_token is false' do
      subject(:sync_seat_link) do
        described_class.new.perform('2020-01-01T01:20:12+02:00', '123', 5, 4, false)
      end

      it 'does not perform Cloud Connector access data sync' do
        expect(CloudConnector::SyncServiceTokenWorker).not_to receive(:perform_async)

        sync_seat_link
      end
    end

    context 'when refresh_token is true' do
      subject(:sync_seat_link) do
        described_class.new.perform('2020-01-01T01:20:12+02:00', '123', 5, 4, true)
      end

      it 'performs Cloud Connector access data sync' do
        expect(CloudConnector::SyncServiceTokenWorker).to receive(:perform_async).with(
          license_id: License.current.id
        )

        sync_seat_link
      end
    end

    context 'when the request is not successful' do
      let(:body) { { success: false, error: "Bad Request" } }

      before do
        stub_request(:post, seat_link_url)
          .to_return_json(status: 400, body: body)
      end

      it 'does not call CloudConnector::SyncServiceTokenWorker' do
        expect(CloudConnector::SyncServiceTokenWorker).not_to receive(:perform_async)

        expect { sync_seat_link }.to raise_error(
          described_class::RequestError,
          'HTTP status code: 400'
        )
      end
    end

    shared_examples 'unsuccessful request' do
      context 'when the request is not successful' do
        before do
          stub_request(:post, seat_link_url)
            .to_return_json(status: 400, body: { success: false, error: "Bad Request" })
        end

        it 'raises an error with the expected message' do
          expect { sync_seat_link }.to raise_error(
            described_class::RequestError,
            'HTTP status code: 400'
          )
        end
      end
    end

    it_behaves_like 'unsuccessful request'
  end

  describe 'sidekiq_retry_in_block' do
    it 'is at least 1 hour in the first retry' do
      expect(
        described_class.sidekiq_retry_in_block.call(0, nil)
      ).to be >= 1.hour
    end
  end
end
