# frozen_string_literal: true

module API
  class VulnerabilityFindings < ::API::Base
    include PaginationParams
    include ::Gitlab::Utils::StrongMemoize

    feature_category :vulnerability_management
    urgency :low

    helpers do
      def pipeline
        strong_memoize(:pipeline) do
          if params[:pipeline_id]
            user_project.all_pipelines.id_in(params[:pipeline_id]).first
          else
            user_project.latest_ingested_security_pipeline
          end
        end
      end

      def vulnerability_findings
        @vulnerability_findings ||= begin
          return paginate(Kaminari.paginate_array([])) unless pipeline

          paginate(findings.page(page).per(per_page).without_count, without_count: true)
        end
      end

      def findings
        ::Security::FindingsFinder.new(pipeline, params: finder_params)
          .execute
          .tap { |findings| findings.each(&:remediations) } # initiates Batchloader
      end

      def page
        params[:page] || 1
      end

      def per_page
        params[:per_page] || 20
      end

      def finder_params
        declared_params.merge(
          limit: page * per_page
        )
      end
    end

    before do
      authenticate!
    end

    params do
      requires :id, types: [String, Integer], desc: 'The ID or URL-encoded path of the project'
    end
    resource :projects, requirements: API::NAMESPACE_OR_PROJECT_REQUIREMENTS do
      params do
        optional :report_type,
          type: Array[String],
          coerce_with: ::API::Validations::Types::CommaSeparatedToArray.coerce,
          desc: 'The type of report vulnerability belongs to',
          values: ::Vulnerabilities::Finding.report_types.keys,
          default: ::Vulnerabilities::Finding.report_types.keys

        optional :scope,
          type: String,
          desc: 'Return vulnerabilities for the given scope: `dismissed` or `all`',
          default: 'dismissed',
          values: %w[all dismissed]

        optional :severity,
          type: Array[String],
          coerce_with: ::API::Validations::Types::CommaSeparatedToArray.coerce,
          desc: 'Returns vulnerabilities belonging to specified severity level: '\
                '`info`, `unknown`, `low`, `medium`, `high`, or `critical`. Defaults to all',
          values: ::Vulnerabilities::Finding.severities.keys,
          default: ::Vulnerabilities::Finding.severities.keys

        optional :pipeline_id, type: String, desc: 'The ID of the pipeline'

        use :pagination
      end
      desc 'Get a list of project vulnerability findings' do
        is_array true
        success ::Vulnerabilities::FindingEntity
      end
      get ':id/vulnerability_findings' do
        authorize! :read_security_resource, user_project

        present vulnerability_findings,
          with: ::Vulnerabilities::FindingEntity,
          request: GrapeRequestProxy.new(request, current_user)
      end
    end
  end
end
