# frozen_string_literal: true

require 'spec_helper'

RSpec.describe Projects::Security::VulnerabilityReportController, feature_category: :vulnerability_management do
  let_it_be(:group)   { create(:group) }
  let_it_be(:project, reload: true) { create(:project, :repository, :public, namespace: group) }
  let_it_be(:user)    { create(:user) }

  before do
    group.add_developer(user)
    stub_licensed_features(security_dashboard: true)
  end

  describe 'GET #index' do
    let(:pipeline) { create(:ci_pipeline, sha: project.commit.id, project: project, user: user) }

    render_views

    subject(:show_security_dashboard) do
      sign_in(user)
      get :index, params: { namespace_id: project.namespace, project_id: project }
    end

    context 'when page is loaded' do
      it_behaves_like 'internal event tracking' do
        let(:event) { 'visit_vulnerability_report' }
        let(:category) { described_class.name }

        subject { show_security_dashboard }
      end
    end

    it_behaves_like 'security and compliance feature'

    context 'when project has no vulnerabilities' do
      it 'renders empty state' do
        show_security_dashboard

        expect(response).to have_gitlab_http_status(:ok)
        expect(response).to render_template(:index)
        expect(response.body).to have_css('div#js-security-report-app[data-has-vulnerabilities="false"]')
      end
    end

    context 'when project has vulnerabilities' do
      before do
        create(:vulnerability, project: project)
      end

      it 'renders dashboard with vulnerability metadata' do
        show_security_dashboard

        expect(response).to have_gitlab_http_status(:ok)
        expect(response).to render_template(:index)
        expect(response.body).to have_css('div#js-security-report-app[data-has-vulnerabilities="true"]')
      end
    end

    it_behaves_like 'tracks govern usage event', 'users_visiting_security_vulnerability_report' do
      let(:request) { show_security_dashboard }
    end

    context "when resolveVulnerabilityWithAi ability is allowed" do
      before do
        allow(Ability).to receive(:allowed?).and_call_original
        allow(Ability).to receive(:allowed?).with(user, :resolve_vulnerability_with_ai, project).and_return(true)

        show_security_dashboard
      end

      it 'sets the frontend ability to true when allowed' do
        expect(response.body).to have_pushed_frontend_ability(resolveVulnerabilityWithAi: true)
      end
    end

    context "when resolveVulnerabilityWithAi ability is not allowed" do
      before do
        allow(Ability).to receive(:allowed?).and_call_original
        allow(Ability).to receive(:allowed?).with(user, :resolve_vulnerability_with_ai, project).and_return(false)

        show_security_dashboard
      end

      it 'sets the frontend ability to false not allowed' do
        expect(response.body).to have_pushed_frontend_ability(resolveVulnerabilityWithAi: false)
      end
    end
  end
end
