import { GlCard, GlCollapse, GlButton } from '@gitlab/ui';
import Vue, { nextTick } from 'vue';
import VueApollo from 'vue-apollo';
import SectionLayout from '~/vue_shared/security_configuration/components/section_layout.vue';
import vulnerabilityArchivesQuery from 'ee/security_configuration/graphql/vulnerability_archives.query.graphql';
import { shallowMountExtended } from 'helpers/vue_test_utils_helper';
import waitForPromises from 'helpers/wait_for_promises';
import createMockApollo from 'helpers/mock_apollo_helper';
import VulnerabilityArchives from 'ee/security_configuration/components/vulnerability_archives.vue';
import VulnerabilityArchivesTable from 'ee/security_configuration/components/vulnerability_archives_table.vue';
import mockData from './vulnerability_archives_mock_data';

Vue.use(VueApollo);

describe('Vulnerability Archive component', () => {
  const projectFullPath = 'flightjs/security-reports';

  let wrapper;

  const defaultHandler = () => jest.fn().mockResolvedValue(mockData);
  const findSectionLayout = () => wrapper.findComponent(SectionLayout);
  const findCards = () => wrapper.findAllComponents(GlCard);
  const findAllGlCollapses = () => wrapper.findAllComponents(GlCollapse);
  const findYearButton = (year) =>
    wrapper.findAllComponents(GlButton).wrappers.find((w) => w.text() === year.toString());

  const createComponent = (propsData) => {
    wrapper = shallowMountExtended(VulnerabilityArchives, {
      provide: {
        projectFullPath,
      },
      propsData,
      apolloProvider: createMockApollo([[vulnerabilityArchivesQuery, defaultHandler()]]),
      stubs: {
        VulnerabilityArchivesTable,
        GlCard,
      },
    });
  };

  beforeEach(() => {
    createComponent();
  });

  it('contains a section to describe the feature', () => {
    expect(findSectionLayout().props('heading')).toBe('Vulnerability archives');
    expect(
      wrapper
        .findByText(
          'Vulnerabilities are retained in the database for one year after the last update. After one year of inactivity, vulnerabilities are archived on the first of each month. Archives are removed after five years.',
        )
        .exists(),
    ).toBe(true);
  });

  it('fetches the archive data and displays a list view for each year', async () => {
    await waitForPromises();
    const cards = findCards();

    expect(cards.length).toBe(4);

    const expectedYears = [2025, 2024, 2023, 2022];
    const expectedCounts = [142, 643, 607, 530];

    cards.wrappers.forEach((card, index) => {
      const text = card.text();
      expect(text).toContain('Download all');
      expect(text).toContain(expectedYears[index].toString());
      expect(text).toContain(expectedCounts[index].toString());
    });
  });

  it('displays the archive table when the year button is clicked', async () => {
    await waitForPromises();

    const collapses = findAllGlCollapses();

    expect(collapses).toHaveLength(4);

    collapses.wrappers.forEach((collapse) => {
      expect(collapse.props('visible')).toBe(false);
    });

    findYearButton(2025).vm.$emit('click');

    await nextTick();

    expect(collapses.at(0).props('visible')).toBe(true);
    expect(collapses.at(1).props('visible')).toBe(false);
    expect(collapses.at(2).props('visible')).toBe(false);
    expect(collapses.at(3).props('visible')).toBe(false);
  });

  it('should have the vulnerability archive table mounted for each row', async () => {
    await waitForPromises();

    const collapses = findAllGlCollapses();

    expect(collapses).toHaveLength(4);

    collapses.wrappers.forEach((w) => {
      expect(w.findComponent(VulnerabilityArchivesTable).exists()).toBe(true);
    });
  });
});
