import { GlFormGroup, GlCollapsibleListbox } from '@gitlab/ui';
import { shallowMount } from '@vue/test-utils';
import VulnerabilityDissmisalReason from 'ee/security_dashboard/components/pipeline/vulnerability_dismissal_reason.vue';
import HelpPopover from '~/vue_shared/components/help_popover.vue';
import { DISMISSAL_REASONS } from 'ee/vulnerabilities/constants';
import { dismissalDescriptions } from 'ee_jest/vulnerabilities/mock_data';

describe('VulnerabilityDissmisalReason', () => {
  let wrapper;

  const createWrapper = (propsData) => {
    wrapper = shallowMount(VulnerabilityDissmisalReason, {
      propsData,
      provide: { dismissalDescriptions },
      stubs: { GlFormGroup },
    });
  };

  const findFormGroup = () => wrapper.findComponent(GlFormGroup);
  const findListBox = () => wrapper.findComponent(GlCollapsibleListbox);
  const findHelpPopover = () => wrapper.findComponent(HelpPopover);

  beforeEach(() => {
    createWrapper();
  });

  it('renders a form group', () => {
    const formGroup = findFormGroup();

    expect(formGroup.text()).toContain('Dismissal reason (required)');
    expect(formGroup.attributes('invalidfeedback')).toBe('A dismissal reason is required.');
  });

  it('renders a help popover with dismissal descriptions', () => {
    const dismissalDescriptionsMessage = Object.entries(DISMISSAL_REASONS)
      .map(([value, text]) => `${text} ${dismissalDescriptions[value]}`)
      .join('');

    expect(findHelpPopover().text()).toBe(dismissalDescriptionsMessage);
  });

  it('renders a listbox', () => {
    const listBox = findListBox();

    expect(listBox.props('items')).toMatchObject(
      Object.entries(DISMISSAL_REASONS).map(([value, text]) => ({ text, value })),
    );
    expect(listBox.props('category')).toBe('secondary');
  });

  it('renders placeholder toggle text when no value', () => {
    expect(findListBox().props('toggleText')).toBe('Select dismissal reason');
  });

  it('renders dismissal reason toggle text when value is given', () => {
    createWrapper({ value: 'acceptable_risk' });

    expect(findListBox().props('toggleText')).toBe('Acceptable risk');
  });

  it('passes the value to listbox', () => {
    createWrapper({ value: 'acceptable_risk' });

    expect(findListBox().props('selected')).toBe('acceptable_risk');
  });

  it('passes the dismissal reasons to the listbox', () => {
    const dismissalReasonItems = Object.entries(DISMISSAL_REASONS).map(([value, text]) => ({
      value,
      text,
    }));
    expect(findListBox().props('items')).toMatchObject(dismissalReasonItems);
  });

  describe('invalid state', () => {
    it.each`
      value                | state    | isInvalid | stateAttribute
      ${null}              | ${true}  | ${false}  | ${'true'}
      ${'acceptable_risk'} | ${true}  | ${false}  | ${'true'}
      ${'acceptable_risk'} | ${false} | ${false}  | ${'true'}
      ${null}              | ${false} | ${true}   | ${undefined}
    `(
      'is $isInvalid when value is $value and showError is $showError',
      ({ value, state, stateAttribute, isInvalid }) => {
        createWrapper({ value, state });

        expect(findFormGroup().attributes('state')).toBe(stateAttribute);
        expect(findListBox().props('variant')).toBe(isInvalid ? 'danger' : 'default');
      },
    );
  });

  it('emits input event when selecting a reason', () => {
    findListBox().vm.$emit('select', 'acceptable_risk');

    expect(wrapper.emitted('input')).toMatchObject([['acceptable_risk']]);
  });
});
