import Vue, { nextTick } from 'vue';
import { GlCollapsibleListbox, GlFormGroup } from '@gitlab/ui';
import VueApollo from 'vue-apollo';
import { shallowMountExtended } from 'helpers/vue_test_utils_helper';
import SelectProjectDropdown from 'ee/security_dashboard/components/shared/vulnerability_report/select_project_dropdown.vue';
import createMockApollo from 'helpers/mock_apollo_helper';
import getGroupProjects from 'ee/security_dashboard/graphql/queries/get_group_projects.query.graphql';
import waitForPromises from 'helpers/wait_for_promises';

describe('Select Project Dropdown component', () => {
  let wrapper;
  let requestHandler;

  const defaultFullPath = 'gitlab-org';

  const defaultNodes = [
    { id: '1', name: 'project-1' },
    { id: '2', name: 'project-2' },
  ];

  const defaultPageInfo = {
    __typename: 'PageInfo',
    hasNextPage: false,
    hasPreviousPage: false,
    startCursor: null,
    endCursor: 'end-cursor',
  };

  const mockApolloHandlers = (nodes = defaultNodes, hasNextPage = false) => {
    return jest.fn().mockResolvedValue({
      data: {
        id: '1',
        group: {
          id: '2',
          projects: {
            nodes,
            pageInfo: { ...defaultPageInfo, hasNextPage },
          },
        },
      },
    });
  };

  const createMockApolloProvider = (handler) => {
    Vue.use(VueApollo);

    requestHandler = handler;
    return createMockApollo([[getGroupProjects, requestHandler]]);
  };

  const findDropdown = () => wrapper.findComponent(GlCollapsibleListbox);
  const findFormGroup = () => wrapper.findComponent(GlFormGroup);
  const selectProjectById = (id) => findDropdown(id).vm.$emit('select', id);

  const createComponent = ({ handlers = mockApolloHandlers(), props = {} } = {}) => {
    wrapper = shallowMountExtended(SelectProjectDropdown, {
      apolloProvider: createMockApolloProvider(handlers),
      provide: {
        fullPath: defaultFullPath,
      },
      propsData: {
        valid: true,
        ...props,
      },
      stubs: { GlCollapsibleListbox },
    });
  };

  beforeEach(async () => {
    createComponent();
    await waitForPromises();
  });

  describe('default', () => {
    it('renders dropdown and form group', () => {
      expect(findDropdown().exists()).toBe(true);
      expect(findDropdown().props()).toMatchObject({ variant: 'default', category: 'primary' });
      expect(findDropdown().attributes('aria-invalid')).toBeUndefined();
      expect(findFormGroup().exists()).toBe(true);
      expect(findFormGroup().attributes('state')).toBe('true');
    });

    it('shows placeholder toggle text', () => {
      expect(findDropdown().props('toggleText')).toBe('Select project');
    });

    it('calls group projects query', () => {
      expect(requestHandler).toHaveBeenCalledWith({
        fullPath: defaultFullPath,
      });
    });

    it('sets correct items', () => {
      expect(findDropdown().props('items')).toMatchObject(
        defaultNodes.map((node) => ({ value: node.id, text: node.name })),
      );
    });
  });

  describe('when valid is `false`', () => {
    beforeEach(() => {
      createComponent({ props: { valid: false } });
    });

    it('sets state correctly for form group', () => {
      expect(findFormGroup().attributes('state')).toBeUndefined();
    });

    it('sets props correctly for dropdown', () => {
      expect(findDropdown().props()).toMatchObject({ variant: 'danger', category: 'secondary' });
      expect(findDropdown().attributes('aria-invalid')).toBe('true');
    });
  });

  describe('when value is set', () => {
    beforeEach(async () => {
      createComponent({ props: { value: '2' } });
      await waitForPromises();
    });

    it('sets correct toggle text', () => {
      expect(findDropdown().props('toggleText')).toBe('project-2');
    });
  });

  describe('when selecting a project', () => {
    const projectId = '2';

    beforeEach(() => {
      selectProjectById(projectId);
    });

    it('emits "input" event', () => {
      expect(wrapper.emitted('input')).toMatchObject([[projectId]]);
    });
  });

  describe('when there is only one page of projects', () => {
    it('sets `infinite-scroll` to `false`', async () => {
      createComponent();
      await waitForPromises();

      expect(findDropdown().props('infiniteScroll')).toBe(false);
    });
  });

  describe('when there is more than a page of projects', () => {
    beforeEach(async () => {
      createComponent({ handlers: mockApolloHandlers(defaultNodes, true) });
      await waitForPromises();
    });

    it('sets `infinite-scroll` to `true`', () => {
      expect(findDropdown().props('infiniteScroll')).toBe(true);
    });

    describe('when the "bottom-reached" event is called', () => {
      beforeEach(async () => {
        createComponent({ handlers: mockApolloHandlers([], true) });
        await waitForPromises();
      });

      it('makes a query to fetch more projects', () => {
        findDropdown().vm.$emit('bottom-reached');
        expect(requestHandler).toHaveBeenCalledWith({
          fullPath: 'gitlab-org',
          after: 'end-cursor',
        });
      });
    });

    describe('when a query is loading a new page of projects', () => {
      it('renders the loading spinner', async () => {
        createComponent({ handlers: mockApolloHandlers([], true) });
        await waitForPromises();

        findDropdown().vm.$emit('bottom-reached');
        await nextTick();

        expect(findDropdown().props('loading')).toBe(true);
      });
    });

    describe('when searching', () => {
      beforeEach(() => {
        const searchTermWithSpace = ' 1 ';
        findDropdown().vm.$emit('search', searchTermWithSpace);
      });

      it('uses the trimmed terms and filters the items', () => {
        expect(findDropdown().props('items')).toMatchObject([{ text: 'project-1', value: '1' }]);
      });
    });
  });
});
