# frozen_string_literal: true

require 'spec_helper'

RSpec.describe GitlabSchema.types['VulnerabilityLocationCoverageFuzzing'] do
  let_it_be(:project) { create(:project) }
  let_it_be(:user) { create(:user) }
  let_it_be(:pipeline) { create(:ee_ci_pipeline, :with_coverage_fuzzing_report, project: project) }

  let_it_be(:build_1) { create(:ci_build, :success, name: 'coverage_fuzzing', pipeline: pipeline) }
  let_it_be(:build_2) { create(:ci_build, :success, name: 'sast', pipeline: pipeline) }
  let_it_be(:artifact_ds) { create(:ee_ci_job_artifact, :coverage_fuzzing, job: build_1) }
  let_it_be(:artifact_sast) { create(:ee_ci_job_artifact, :sast, job: build_2) }
  let_it_be(:report_ds) { create(:ci_reports_security_report, pipeline: pipeline, type: :coverage_fuzzing) }
  let_it_be(:report_sast) { create(:ci_reports_security_report, pipeline: pipeline, type: :sast) }

  before_all do
    ds_content = File.read(artifact_ds.file.path)
    Gitlab::Ci::Parsers::Security::DependencyScanning.parse!(ds_content, report_ds)
    report_ds.merge!(report_ds)
    sast_content = File.read(artifact_sast.file.path)
    Gitlab::Ci::Parsers::Security::Sast.parse!(sast_content, report_sast)
    report_sast.merge!(report_sast)

    findings = { artifact_ds => report_ds, artifact_sast => report_sast }.flat_map do |artifact, report|
      scan = create(:security_scan, :latest_successful, scan_type: artifact.job.name, build: artifact.job)
      scanner_external_id = report.scanners.each_value.first.external_id
      scanner = create(:vulnerabilities_scanner, project: pipeline.project, external_id: scanner_external_id)

      report.findings.collect do |finding|
        create(
          :security_finding,
          :with_finding_data,
          severity: finding.severity,
          uuid: finding.uuid,
          deduplicated: true,
          scan: scan,
          scanner: scanner
        )
      end
    end

    findings.second.update!(deduplicated: false)
  end

  before do
    stub_licensed_features(coverage_fuzzing: true, security_dashboard: true)

    project.add_developer(user)
  end

  it do
    expect(described_class).to have_graphql_fields(
      :vulnerable_class,
      :end_line,
      :file,
      :vulnerable_method,
      :start_line,
      :blob_path,
      :crash_type,
      :crash_address,
      :stacktrace_snippet
    )
  end

  subject { GitlabSchema.execute(query, context: { current_user: user }).as_json }

  specify { expect(described_class.graphql_name).to eq('VulnerabilityLocationCoverageFuzzing') }

  describe 'checking field contents' do
    let(:query) do
      %(
        query {
          project(fullPath: "#{project.full_path}") {
            pipeline(iid: "#{pipeline.iid}") {
              securityReportFindings {
                nodes {
                  location {
                    ... on VulnerabilityLocationCoverageFuzzing {
                      crashType
                      crashAddress
                      stacktraceSnippet
                    }
                  }
                }
              }
            }
          }
        }
      )
    end

    it 'checks the contents of the VulnerabilityLocationCoverageFuzzing fields' do
      vulnerability = subject.dig('data', 'project', 'pipeline', 'securityReportFindings', 'nodes').first

      expect(vulnerability['location']['crashType']).to eq("Heap-buffer-overflow\nREAD 1")
      expect(vulnerability['location']['crashAddress']).to eq("0x602000001573")
      expect(vulnerability['location']['stacktraceSnippet']).to eq("INFO: Seed: 3415817494\nINFO: Loaded 1 modules")
    end
  end
end
